@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2025 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,14 +28,17 @@
 #include "io/path_parse.h"
 #include "io/sequence.h"
 
+#define DM_PATTERN "\\d{4}-\\d{2}-\\d{2}"
+#define DT_PATTERN  "\\d{4}-\\d{2}-\\d{2}T\\d{2}-\\d{2}-\\d{2}"
+
 static void display_path_parse_error(pathparse_errors err, const gchar *addstr) {
 	if (!err) return;
 	gchar *startstr = (err < 0) ? _("Warning code:"): _("Error code:");
 	gchar *endstr = (err < 0) ? _("going on") : _("aborting");
 	gchar *msg = NULL;
 	gchar *color = (err < 0) ? "salmon" : "red";
-	gchar addbuf[FLEN_VALUE];
-	g_snprintf(addbuf, FLEN_VALUE - 1, "%s", (!addstr) ? "" : addstr);
+	gchar addbuf[256];
+	g_snprintf(addbuf, 255, "%s", (!addstr) ? "" : addstr);
 
 	switch (err) {
 		case PATHPARSE_ERR_HEADER_NULL:
@@ -74,9 +77,16 @@ static void display_path_parse_error(pathparse_errors err, const gchar *addstr)
 		case PATHPARSE_ERR_BADSTRING_NOFAIL:
 			msg = _("Wrongly formatted string: ");
 			break;
+		case PATHPARSE_ERR_WILDCARD_SYNTAX:
+		case PATHPARSE_ERR_WILDCARD_SYNTAX_NOFAIL:
+			msg = _("Wrong wildcard usage: ");
+			break;
 		case PATHPARSE_ERR_MORE_THAN_ONE_HIT:
 			msg = _("More than one match for: ");
 			break;
+		case PATHPARSE_ERR_HITS_ALL_NEWER:
+			msg = _("All hits newer than: ");
+			break;
 		case PATHPARSE_ERR_NO_HIT_FOUND:
 			msg = _("No match found for: ");
 			break;
@@ -94,7 +104,7 @@ static void display_path_parse_error(pathparse_errors err, const gchar *addstr)
 	siril_log_color_message("%s %d - %s%s - %s\n", color, startstr, err, msg, addbuf, endstr);
 }
 
-static pathparse_errors read_key_from_header_text(gchar **headers, gchar *key, double *numvalue, gchar *strvalue) {
+pathparse_errors read_key_from_header_text(gchar **headers, gchar *key, double *numvalue, gchar *strvalue) {
 	pathparse_errors status = PATHPARSE_ERR_OK;
 	if (!headers)
 		return PATHPARSE_ERR_HEADER_NULL;
@@ -130,26 +140,88 @@ static pathparse_errors read_key_from_header_text(gchar **headers, gchar *key, d
 	return status;
 }
 
+typedef struct _file_date {
+	GDateTime *date;
+	gchar *filename;
+} file_date;
+
+static file_date *new_file_date(const gchar *date, const gchar *filename, gboolean isdatetime) {
+	file_date *fd = g_slice_new(file_date);
+	gboolean set = FALSE;
+	gint year, month, day, hour, min, sec;
+	GTimeZone *tz = g_time_zone_new_utc();
+	if (isdatetime) {
+		if (sscanf(date, "%d-%d-%dT%d-%d-%d", &year, &month, &day, &hour, &min,	&sec) == 6) {
+			fd->date = g_date_time_new(tz, year, month, day, hour, min, (double)sec);
+			set = TRUE;
+		}
+	} else {
+		if (sscanf(date, "%d-%d-%d", &year, &month, &day) == 3) {
+			fd->date = g_date_time_new(tz, year, month, day, 0, 0, 0.);
+			set = TRUE;
+		}
+	}
+	g_time_zone_unref(tz);
+	if (!set) {
+		g_slice_free(file_date, fd);
+		return NULL;
+	}
+	fd->filename = g_strdup(filename);
+	return fd;
+}
+
+static gint file_date_compare(const file_date *a, const file_date *b) {
+	int res = g_date_time_compare(a->date, b->date);
+	if (!res)
+		return (strlen(a->filename) == 1) ? 1 : -1; // we want filename "." (the ref file), to be after
+	return res;
+}
+
+static gint file_date_match_file(const file_date *a, gchar *data) {
+	return g_strcmp0(a->filename, data);
+}
+
+static void file_date_free(gpointer data) {
+	file_date *fd = (file_date *)data;
+	g_free(fd->filename);
+	g_date_time_unref(fd->date);
+	g_slice_free(file_date, fd);
+}
+
 /*
 This function takes an expression with potentially a wildcard in it
 Searches the directory for files matching the pattern
 and returns the first occurence of the match if any
 */
-static gchar *wildcard_check(gchar *expression, int *status) {
+static gchar *wildcard_check(gchar *expression, int *status, gchar *target_date, gboolean isdatetime) {
 	gchar *out = NULL, *dirname = NULL, *basename = NULL, *currfile = NULL;
 	const gchar *file = NULL;
 	GDir *dir;
 	GError *error = NULL;
-	GPatternSpec *patternspec;
 	gint count = 0;
 	struct stat fileInfo, currfileInfo;
+	GList *fds = NULL;
 
-	if (!g_utf8_strchr(expression, -1, '*')) {
-		return g_strdup(expression);
+	//we need to check that the folder path does not contain wildcards
+	GString *newexp = g_string_new(expression);
+	g_string_replace(newexp, (isdatetime) ? DT_PATTERN : DM_PATTERN, "[DATE]", 0);
+	gchar *tmpexp = g_string_free(newexp, FALSE);
+	gchar *tmpdirname = g_path_get_dirname(tmpexp);
+	gchar *tmpbasename = g_path_get_basename(tmpexp);
+	if (g_strstr_len(tmpdirname, -1, "[DATE]") || g_strstr_len(tmpdirname, -1, "*")) {
+		*status = PATHPARSE_ERR_WILDCARD_SYNTAX;
+		display_path_parse_error(*status, tmpdirname);
+		g_free(tmpexp);
+		g_free(tmpdirname);
+		g_free(tmpbasename);
+		return NULL;
 	}
-
-	dirname = g_path_get_dirname(expression);
-	basename = g_path_get_basename(expression);
+	GString *newdir = g_string_new(tmpdirname);
+	g_string_replace(newdir, "[DATE]", (isdatetime) ? DT_PATTERN : DM_PATTERN, 0);
+	dirname =  g_string_free(newdir, FALSE);
+	GString *newbase = g_string_new(tmpbasename);
+	g_string_replace(newbase, "[DATE]", (isdatetime) ? DT_PATTERN : DM_PATTERN, 0);
+	basename =  g_string_free(newbase, FALSE);
 
 	if ((dir = g_dir_open(dirname, 0, &error)) == NULL) {
 		siril_debug_print("wildcard dircheck: %s\n", error->message);
@@ -158,36 +230,69 @@ static gchar *wildcard_check(gchar *expression, int *status) {
 		g_clear_error(&error);
 		g_free(dirname);
 		g_free(basename);
+		g_free(tmpdirname);
+		g_free(tmpbasename);
 		return out;
 	}
-	patternspec = g_pattern_spec_new(basename);
+
+	if (target_date) { // Init with target_date
+		fds = g_list_append(fds, new_file_date(target_date, ".", isdatetime));
+	}
+
 	while ((file = g_dir_read_name(dir)) != NULL) {
-#if GLIB_CHECK_VERSION(2,70,0)
-		if (g_pattern_spec_match(patternspec, strlen(file), file, NULL)) {
-#else
-		if (g_pattern_match(patternspec, strlen(file), file, NULL)) {
-#endif
-			if (!count) {
-				out = g_build_filename(dirname, file, NULL);
-					if (stat(out, &fileInfo))
-						siril_debug_print("stat() failed\n");
-			} else {
-				currfile = g_build_filename(dirname, file, NULL);
-				if (stat(currfile, &currfileInfo))
-						siril_debug_print("stat() failed\n");
-				if (currfileInfo.st_ctime > fileInfo.st_ctime) { // currfile is more recent
+		if (g_regex_match_simple(basename, file, G_REGEX_RAW, 0)) {
+			if (!target_date) { // No date_obs we fetch the most recent file based on stat
+				if (!count) {
 					out = g_build_filename(dirname, file, NULL);
-					if (stat(out, &fileInfo))
-						siril_debug_print("stat() failed\n");
+						if (stat(out, &fileInfo))
+							siril_debug_print("stat() failed\n");
+				} else {
+					currfile = g_build_filename(dirname, file, NULL);
+					if (stat(currfile, &currfileInfo))
+							siril_debug_print("stat() failed\n");
+					if (currfileInfo.st_ctime > fileInfo.st_ctime) { // currfile is more recent
+						out = g_build_filename(dirname, file, NULL);
+						if (stat(out, &fileInfo))
+							siril_debug_print("stat() failed\n");
+					}
 				}
+				count++;
+			} else {
+				out = g_build_filename(dirname, file, NULL);
+				GRegex *regex = g_regex_new((isdatetime) ? DT_PATTERN : DM_PATTERN, 0, 0, NULL);
+				GMatchInfo *match_info;
+				g_regex_match(regex, out, 0, &match_info);
+				if (g_match_info_matches(match_info)) {
+					gchar *date = g_match_info_fetch(match_info, 0);
+					siril_debug_print("Captured date: %s\n", date);
+					fds = g_list_append(fds, new_file_date(date, out, isdatetime));
+					g_free(date);
+					count++;
+				}
+				g_free(out);
+				out = NULL;
+				g_match_info_free(match_info);
+				g_regex_unref(regex);
 			}
-			count++;
 		}
 	}
-	if (count > 1) {
+
+	if (!target_date && count > 1) {
 		*status = PATHPARSE_ERR_MORE_THAN_ONE_HIT;
 		display_path_parse_error(*status, basename);
 		siril_log_color_message(_("Using most recent matching file: %s\n"), "salmon", out);
+	} else if (target_date && count >= 1) { // we sort the date_file list
+		fds = g_list_sort(fds, (GCompareFunc)file_date_compare);
+		GList *ref = g_list_find_custom(fds, ".", (GCompareFunc)file_date_match_file);
+		gint refpos = g_list_position(fds, ref);
+		if (refpos == 0) { // all files matching have newer DATE-OBS
+			out = g_strdup(((file_date *)g_list_nth_data(fds, 1))->filename);
+			*status = PATHPARSE_ERR_HITS_ALL_NEWER;
+			display_path_parse_error(*status, basename);
+			siril_log_color_message(_("Using the closest matching file: %s\n"), "salmon", out);
+		} else { // we return the file which has DATE-OBS lteq to reference
+			out = g_strdup(((file_date *)g_list_nth_data(fds, refpos - 1))->filename);
+		}
 	} else if (!count) {
 		*status = PATHPARSE_ERR_NO_HIT_FOUND;
 		display_path_parse_error(*status, expression);
@@ -196,7 +301,9 @@ static gchar *wildcard_check(gchar *expression, int *status) {
 	g_free(dirname);
 	g_free(basename);
 	g_free(currfile);
-	g_pattern_spec_free(patternspec);
+	g_free(tmpdirname);
+	g_free(tmpbasename);
+	g_list_free_full(fds, (GDestroyNotify)file_date_free);
 	return out;
 }
 
@@ -216,6 +323,9 @@ In write mode "nofail", it will try to return something no matter what
 */
 gchar *path_parse(fits *fit, const gchar *expression, pathparse_mode mode, int *status) {
 	*status = PATHPARSE_ERR_OK;
+	gboolean has_wildcard = FALSE;
+	gchar *target_date = NULL; // for date wildcard in READMODE
+	gboolean isdatetime = FALSE; // for date wildcard in READMODE
 	if (!g_utf8_strchr(expression, -1, '$')) { // nothing to parse, return original string
 		return g_strdup(expression);
 	}
@@ -228,6 +338,16 @@ gchar *path_parse(fits *fit, const gchar *expression, pathparse_mode mode, int *
 			return out;
 		}
 	}
+
+	// Checking that we don't have 2 date with wildcards, this is not allowed
+	if (count_pattern_occurence(expression, "\\$\\*DATE") > 1) {
+		*status = nofail * PATHPARSE_ERR_WILDCARD_SYNTAX;
+		if (*status > 0) {
+			display_path_parse_error(*status, expression);
+			return out;
+		}
+	}
+
 	if (g_str_has_prefix(expression, "$def")) { // using reserved keywords $defbias, $defdark, $defflat, $defstack
 		if (!g_strcmp0(expression + 4, "bias")) {
 			localexpression = g_strdup(com.pref.prepro.bias_lib);
@@ -237,6 +357,8 @@ gchar *path_parse(fits *fit, const gchar *expression, pathparse_mode mode, int *
 			localexpression = g_strdup(com.pref.prepro.flat_lib);
 		} else if (!g_strcmp0(expression + 4, "stack")) {
 			localexpression = g_strdup(com.pref.prepro.stack_default);
+		} else if (!g_strcmp0(expression + 4, "disto")) {
+			localexpression = g_strdup(com.pref.prepro.disto_lib);
 		} else {
 			*status = nofail * PATHPARSE_ERR_WRONG_RESERVED_KEYWORD;
 			display_path_parse_error(*status, expression);
@@ -257,8 +379,11 @@ gchar *path_parse(fits *fit, const gchar *expression, pathparse_mode mode, int *
 	gchar *pattern = "\\$(.+?)\\$";
 	gchar **tokens = g_regex_split_simple(pattern, localexpression, G_REGEX_RAW, 0);
 	gchar **headerkeys = NULL;
+
 	if (fit->header) { // avoiding split in case we are in nofail and header is empty
-		headerkeys = g_strsplit(fit->header, "\n", 0);
+		gchar *fullheader = g_strconcat(fit->header, "\n", fit->unknown_keys, NULL);
+		headerkeys = g_strsplit(fullheader, "\n", 0);
+		g_free(fullheader);
 	}
 	for (int i = 0; i < g_strv_length(tokens); i++) {
 		if (!tokens[i] || tokens[i][0] == '\0')
@@ -296,9 +421,22 @@ gchar *path_parse(fits *fit, const gchar *expression, pathparse_mode mode, int *
 		}
 		// Check if expression starts with a * wildcard
 		// Behavior will depend if we are in read or write mode
+		gboolean has_wildcard_date = FALSE;
 		if (subs[0][0] == '*') {
 			if (mode == PATHPARSE_MODE_READ) {
-				g_snprintf(buf, 2, "%s", "*");
+				has_wildcard = TRUE;
+				if ((g_str_has_prefix(subs[1],"dm"))) {
+					g_snprintf(buf, strlen(DM_PATTERN) + 1, "%s", DM_PATTERN);
+					has_wildcard_date = TRUE; // we still need to parse the target date without copying to buf
+					g_snprintf(key, 9, "%s", subs[0] + 1);
+				} else if ((g_str_has_prefix(subs[1],"dt"))) {
+					g_snprintf(buf, strlen(DT_PATTERN) + 1, "%s", DT_PATTERN);
+					has_wildcard_date = TRUE; // we still need to parse the target date without copying to buf
+					g_snprintf(key, 9, "%s", subs[0] + 1);
+					isdatetime = TRUE;
+				} else {
+					g_snprintf(buf, 2, "%s", "*");
+				}
 			} else {
 				g_snprintf(key, 9, "%s", subs[0] + 1);
 			}
@@ -309,7 +447,7 @@ gchar *path_parse(fits *fit, const gchar *expression, pathparse_mode mode, int *
 			printf("Wildcard in READ mode\n");
 		} else if (subs[1][0] == '%' && (g_str_has_suffix(subs[1], "d") || g_str_has_suffix(subs[1], "f"))) { // case %d or %f
 			gboolean isint = g_str_has_suffix(subs[1], "d");
-			double val;
+			double val = 0.;
 			*status = nofail * read_key_from_header_text(headerkeys, key,&val, NULL);
 			display_path_parse_error(*status, key);
 			if (*status > 0) {
@@ -358,7 +496,7 @@ gchar *path_parse(fits *fit, const gchar *expression, pathparse_mode mode, int *
 			double minus_hour = -1. * g_ascii_strtod(subs[1] + 2, NULL);
 			char val[FLEN_VALUE];
 			if (!headerkeys) {
-				*status = 1;
+				*status = nofail * PATHPARSE_ERR_HEADER_NULL;
 			} else {
 				*status = nofail * read_key_from_header_text(headerkeys, key, NULL, val);
 				display_path_parse_error(*status, key);
@@ -380,7 +518,11 @@ gchar *path_parse(fits *fit, const gchar *expression, pathparse_mode mode, int *
 				if (*status == 0) {
 					GDateTime *read_time_corr = g_date_time_add_hours(read_time, minus_hour);
 					gchar *fmtdate = date_time_to_date(read_time_corr);
-					strncpy(buf, fmtdate, FLEN_VALUE - 1);
+					if (has_wildcard_date) {
+						target_date = g_strdup(fmtdate);
+					} else {
+						strncpy(buf, fmtdate, FLEN_VALUE - 1);
+					}
 					g_date_time_unref(read_time);
 					g_date_time_unref(read_time_corr);
 					g_free(fmtdate);
@@ -410,7 +552,11 @@ gchar *path_parse(fits *fit, const gchar *expression, pathparse_mode mode, int *
 				}
 				if (*status == 0) {
 					gchar *fmtdate = date_time_to_date_time(read_time);
-					strncpy(buf, fmtdate, FLEN_VALUE - 1);
+					if (has_wildcard_date) {
+						target_date = g_strdup(fmtdate);
+					} else {
+						strncpy(buf, fmtdate, FLEN_VALUE - 1);
+					}
 					g_date_time_unref(read_time);
 					g_free(fmtdate);
 				}
@@ -488,8 +634,8 @@ gchar *path_parse(fits *fit, const gchar *expression, pathparse_mode mode, int *
 	out = g_strjoinv("", tokens);
 	siril_debug_print("String in: %s\n", expression);
 	siril_debug_print("String out: %s\n", out);
-	if (mode == PATHPARSE_MODE_READ) {
-		gchar *foundmatch = wildcard_check(out, status);
+	if (has_wildcard) {
+		gchar *foundmatch = wildcard_check(out, status, target_date, isdatetime);
 		g_free(out);
 		out = NULL;
 		if (*status <= 0) out = g_strdup(foundmatch);
@@ -514,41 +660,16 @@ gchar *update_header_and_parse(fits *fit, gchar *expression, pathparse_mode mode
 	if (!g_utf8_strchr(expression, -1, '$')) { // nothing to parse, return original string
 		return g_strdup(expression);
 	}
-	fits tmpfit = { 0 };
-	fitsfile *fptr;
-	int fstatus = 0;
 	gchar *parsedname = NULL, *dirname = NULL;
-	copyfits(fit, &tmpfit, CP_FORMAT, 0);
-	copy_fits_metadata(fit, &tmpfit); // otherwise some fields get wiped out like date-obs
-	const gchar *tmpdir = g_get_tmp_dir();
-	gchar *tmpheadername = g_build_filename(tmpdir, "header.fit", NULL);
-	unlink(tmpheadername);
-	fits_create_diskfile(&fptr, tmpheadername, &fstatus);
-	if (fstatus) {
-		char tbuf[30];
-		fits_get_errstatus(fstatus, tbuf);
-		*status = PATHPARSE_ERR_TMPFIT;
-		display_path_parse_error(*status, tbuf);
-		goto free_and_exit;
-	}
-	tmpfit.fptr = fptr;
-	save_fits_header(&tmpfit);
-	tmpfit.header = copy_header(&tmpfit);
-	parsedname = path_parse(&tmpfit, expression, mode, status);
-	fits_close_file(fptr, &fstatus);
-
+	update_fits_header(fit);
+	parsedname = path_parse(fit, expression, mode, status);
 	if (parsedname && createdir) {
 		dirname = g_path_get_dirname(parsedname);
-		if (g_mkdir_with_parents(dirname, 0755) < 0) {
-			siril_log_color_message(_("Cannot create output folder: %s\n"), "red", dirname);
+		if (siril_mkdir_with_parents(dirname, 0755) < 0) {
 			g_free(parsedname);
 			parsedname = NULL;
 		}
 	}
-
-free_and_exit:
-	g_free(tmpheadername);
 	g_free(dirname);
-	clearfits(&tmpfit);
 	return parsedname;
 }
