@@ -1,8 +1,8 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
- * Reference site is https://free-astro.org/index.php/Siril
+ * Copyright (C) 2012-2025 team free-astro (see more in AUTHORS file)
+ * Reference site is https://siril.org
  *
  * Siril is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,7 +23,11 @@
 #  include <config.h>
 #endif
 
+// #define DEBUG_MAIN
+
+#include <gsl/gsl_errno.h>
 #include <gtk/gtk.h>
+#include <gtksourceview/gtksource.h>
 #include <stdio.h>
 #include <string.h>
 #include <locale.h>
@@ -40,33 +44,48 @@
 #include <windows.h>
 #endif
 
+#ifdef HAVE_LIBCURL
+#include <curl/curl.h>
+#endif
+
+#ifdef _OPENMP
+#include <omp.h>
+#endif
+
+#ifdef HAVE_LIBGIT2
+#include <git2.h>
+#endif
+
+#include "siril_resource.h"
 #include "git-version.h"
 #include "core/siril.h"
+#include "core/icc_profile.h"
 #include "core/proto.h"
+#include "algos/siril_random.h"
 #include "core/siril_actions.h"
 #include "core/initfile.h"
 #include "core/command_line_processor.h"
-#include "core/command.h"
 #include "core/pipe.h"
 #include "core/signals.h"
 #include "core/siril_app_dirs.h"
 #include "core/siril_language.h"
+#include "core/siril_networking.h"
+#include "io/siril_pythonmodule.h"
 #include "core/siril_update.h"
 #include "core/siril_log.h"
 #include "core/OS_utils.h"
-#include "algos/star_finder.h"
 #include "io/sequence.h"
 #include "io/conversion.h"
 #include "io/single_image.h"
+#include "gui/ui_files.h"
 #include "gui/utils.h"
 #include "gui/callbacks.h"
-#include "gui/progress_and_log.h"
 #include "gui/siril_css.h"
-#include "registration/registration.h"
+
 
 /* the global variables of the whole project */
-cominfo com;	// the core data struct
-guiinfo gui;	// the gui data struct
+cominfo com = { 0 };	// the core data struct
+guiinfo gui = { 0 };	// the gui data struct
 fits gfit;	// currently loaded image
 
 static gchar *main_option_directory = NULL;
@@ -101,14 +120,21 @@ static gboolean _print_list_of_formats_and_exit(const gchar *option_name,
 	return TRUE;
 }
 
+static gboolean _set_offline(const gchar *option_name,
+		const gchar *value, gpointer data, GError **error) {
+	set_online_status(FALSE);
+	return TRUE;
+}
+
 static GOptionEntry main_option[] = {
 	{ "directory", 'd', 0, G_OPTION_ARG_FILENAME, &main_option_directory, N_("changing the current working directory as the argument"), NULL },
 	{ "script", 's', 0, G_OPTION_ARG_FILENAME, &main_option_script, N_("run the siril commands script in console mode. If argument is equal to \"-\", then siril will read stdin input"), NULL },
 	{ "initfile", 'i', 0, G_OPTION_ARG_FILENAME, &main_option_initfile, N_("load configuration from file name instead of the default configuration file"), NULL },
 	{ "pipe", 'p', 0, G_OPTION_ARG_NONE, &main_option_pipe, N_("run in console mode with command and log stream through named pipes"), NULL },
 	{ "inpipe", 'r', 0, G_OPTION_ARG_FILENAME, &main_option_rpipe_path, N_("specify the path for the read pipe, the one receiving commands"), NULL },
-	{ "outpipe", 'w', 0, G_OPTION_ARG_FILENAME, &main_option_wpipe_path, N_("specify the path for the write pipe, the one outputing messages"), NULL },
+	{ "outpipe", 'w', 0, G_OPTION_ARG_FILENAME, &main_option_wpipe_path, N_("specify the path for the write pipe, the one outputting messages"), NULL },
 	{ "format", 'f', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, _print_list_of_formats_and_exit, N_("print all supported image file formats (depending on installed libraries)" ), NULL },
+	{ "offline", 'o', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, _set_offline, N_("start in offline mode"), NULL },
 	{ "version", 'v', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, _print_version_and_exit, N_("print the application’s version"), NULL},
 	{ "copyright", 'c', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, _print_copyright_and_exit, N_("print the copyright"), NULL},
 	{ NULL },
@@ -123,27 +149,95 @@ static GActionEntry app_entries[] = {
 	{ "about", about_action_activate }
 };
 
+gboolean builder_add_from_resource_with_replace(GtkBuilder *builder, const gchar* resource_path, GError** error) {
+	// Load the resource data
+	GBytes* resource_data = g_resources_lookup_data(resource_path, G_RESOURCE_LOOKUP_FLAGS_NONE, error);
+	if (!resource_data) {
+		return FALSE;
+	}
 
-void load_glade_file() {
+	// Get the data as a string
+	gsize data_size;
+	const gchar* data = g_bytes_get_data(resource_data, &data_size);
+#ifdef __APPLE__
+	// Create a GString from the data
+	GString *str = g_string_new_len(data, data_size);
+
+	// Perform the replacement - no limit on number of replacements
+	g_string_replace(str, "GDK_CONTROL_MASK", "GDK_META_MASK", 0);
+
+	// Get the modified result
+	const gchar *result = str->str;
+#else
+	const gchar* result = data;
+#endif
+
+	// Create and load the builder
+	gboolean success = gtk_builder_add_from_string(builder, result, -1, error);
+
+	// Clean up
+#ifdef __APPLE__
+	g_string_free(str, TRUE);  // TRUE to free the string data as well
+#endif
+	g_bytes_unref(resource_data);
+
+	if (!success) {
+		g_object_unref(builder);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+void load_ui_files() {
 	GError *err = NULL;
-	gchar* gladefile;
 	gboolean retval;
 
-	gladefile = g_build_filename(siril_get_system_data_dir(), GLADE_FILE, NULL);
-
-	/* try to load the glade file, from the sources defined above */
-	gui.builder = gtk_builder_new();
-	// TODO: the following gtk_builder_add_from_file call is the source
-	// of libfontconfig memory leaks.
-	retval = gtk_builder_add_from_file(gui.builder, gladefile, &err);
-	if (!retval) {
+	/* try to load the first UI file, from the sources defined above */
+	gui.builder = gtk_builder_new_from_resource(ui_files[0]);
+	if (!gui.builder) {
 		g_error(_("%s was not found or contains errors, "
-					"cannot render GUI:\n%s\n Exiting.\n"), gladefile, err->message);
-		g_clear_error(&err);
+					"cannot render GUI.\n Exiting.\n"), ui_files[0]);
 		exit(EXIT_FAILURE);
 	}
-	g_print(_("Successfully loaded '%s'\n"), gladefile);
-	g_free(gladefile);
+#ifdef DEBUG_MAIN
+	siril_debug_print("Successfully loaded '%s'\n", ui_files[0]);
+#endif
+
+	uint32_t i = 1;
+	while (*ui_files[i]) {
+
+		/* try to load each successive UI file from ui_files */
+		retval = gtk_builder_add_from_resource(gui.builder, ui_files[i], &err);
+		if (!retval) {
+			g_error(_("%s was not found or contains errors, "
+						"cannot render GUI:\n%s\n Exiting.\n"), ui_files[i], err->message);
+			g_clear_error(&err);
+			exit(EXIT_FAILURE);
+		}
+#ifdef DEBUG_MAIN
+		siril_debug_print("Successfully loaded '%s'\n", ui_files[i]);
+#endif
+		i++;
+	}
+	/* Now we load any UI files that use the primary accelerator.
+	   These get processed specially so that on MacOS GDK_CONTROL_MASK is replaced
+	   with GDK_META_MASK so that shortcuts work with Cmd as expected on that OS. */
+	i = 0;
+	while (*ui_files_with_primary_accelerator[i]) {
+		retval = builder_add_from_resource_with_replace(gui.builder, ui_files_with_primary_accelerator[i], &err);
+		retval = gtk_builder_add_from_resource(gui.builder, ui_files[i], &err);
+		if (!retval) {
+			g_error(_("%s was not found or contains errors, "
+			"cannot render GUI:\n%s\n Exiting.\n"), ui_files[i], err->message);
+			g_clear_error(&err);
+			exit(EXIT_FAILURE);
+		}
+		#ifdef DEBUG_MAIN
+		siril_debug_print("Successfully loaded '%s'\n", ui_files[i]);
+		#endif
+		i++;
+	}
 }
 
 #if defined (HAVE_FFTW3F_THREADS) && (_OPENMP)
@@ -156,25 +250,19 @@ void parallel_loop(void *(*work)(char *), char *jobdata, size_t elsize, int njob
 #endif
 
 static void global_initialization() {
+	gsl_set_error_handler_off();
 	com.star_is_seqdata = FALSE;
 	com.stars = NULL;
 	com.tilt = NULL;
 	com.uniq = NULL;
-	com.child_is_running = EXT_NONE;
 	com.kernel = NULL;
 	com.kernelsize = 0;
 	com.kernelchannels = 0;
-	com.num_gnuplot_handles = 0;
-	com.gnuplot_handles = NULL;
-#ifdef _WIN32
-	com.childhandle = NULL;
-#else
-	com.childpid = 0;
-#endif
+	memset(&com.spcc_data, 0, sizeof(struct spcc_data_store));
 	memset(&com.selection, 0, sizeof(rectangle));
 	memset(com.layers_hist, 0, sizeof(com.layers_hist));
-
 	gui.selected_star = -1;
+	gui.repo_scripts = NULL;
 	gui.qphot = NULL;
 	gui.draw_extra = NULL;
 	gui.cvport = RED_VPORT;
@@ -200,11 +288,17 @@ static void global_initialization() {
 		void *data), void *data);
 #endif
 
+#ifdef _OPENMP
+	omp_set_max_active_levels(2);
+#endif
+
+#ifdef HAVE_LIBGIT2
+	git_libgit2_init();
+#endif
 }
 
 static void siril_app_startup(GApplication *application) {
 	signals_init();
-
 	/*
 	 * Force C locale for numbers to avoid "," being used as decimal separator.
 	 * Called here and not in main() because setlocale(LC_ALL, "") is called as
@@ -212,12 +306,19 @@ static void siril_app_startup(GApplication *application) {
 	 */
 	setlocale(LC_NUMERIC, "C");
 
+#ifdef _WIN32
+	GtkSettings *settings = gtk_settings_get_default();
+	g_object_set(settings, "gtk-im-module", "ime", NULL);
+#endif
+
 	g_set_application_name(PACKAGE_NAME);
 	gtk_window_set_default_icon_name("siril");
 	g_application_set_resource_base_path(application, "/org/siril/Siril/pixmaps/");
 
 	g_action_map_add_action_entries(G_ACTION_MAP(application), app_entries,
 			G_N_ELEMENTS(app_entries), application);
+	// Initialize GtkSource
+	gtk_source_init();
 }
 
 static void siril_app_activate(GApplication *application) {
@@ -226,12 +327,12 @@ static void siril_app_activate(GApplication *application) {
 		com.script = TRUE;
 		com.headless = TRUE;
 	}
+
 #if defined(_WIN32) && !defined(SIRIL_UNSTABLE)
 	ShowWindow(GetConsoleWindow(), SW_MINIMIZE); //hiding the console
 #endif
-
+	siril_initialize_rng();
 	global_initialization();
-
 	/* initialize sequence-related stuff */
 	initialize_sequence(&com.seq, TRUE);
 
@@ -250,7 +351,6 @@ static void siril_app_activate(GApplication *application) {
 	}
 
 	// After this point com.pref is populated
-
 	siril_language_parser_init();
 	if (com.pref.lang)
 		language_init(com.pref.lang);
@@ -278,6 +378,8 @@ static void siril_app_activate(GApplication *application) {
 	}
 
 	init_num_procs();
+	initialize_python_venv_in_thread();
+	initialize_profiles_and_transforms(); // color management
 
 	if (com.headless) {
 		if (main_option_script) {
@@ -311,22 +413,30 @@ static void siril_app_activate(GApplication *application) {
 			read_pipe(main_option_rpipe_path);
 		}
 	}
+
 	if (!com.headless) {
+		/* Load GResource */
+		com.resource = siril_resource_get_resource();
 		/* Load preferred theme */
 		load_prefered_theme(com.pref.gui.combo_theme);
 		/* Load the css sheet for general style */
 		load_css_style_sheet();
-		/* Load glade file */
-		load_glade_file();
+		/* Load UI files */
+		load_ui_files();
 		/* Passing GApplication to the control center */
 		gtk_window_set_application(GTK_WINDOW(GTK_APPLICATION_WINDOW(lookup_widget("control_window"))), GTK_APPLICATION(application));
 		/* Load state of the main windows (position and maximized) */
-		load_main_window_state();
-#if defined(HAVE_JSON_GLIB) && defined(HAVE_NETWORKING)
+		gui_function(load_main_window_state, NULL);
+#if defined(HAVE_LIBCURL)
+		curl_global_init(CURL_GLOBAL_ALL);
 		/* Check for update */
-		if (com.pref.check_update) {
-			siril_check_updates(FALSE);
+		if (is_online()) {
+			if (com.pref.check_update) {
+				siril_check_updates(FALSE);
+			}
+			siril_check_notifications(FALSE);
 		}
+
 #else
 		gtk_widget_set_visible(lookup_widget("main_menu_updates"), FALSE);
 		gtk_widget_set_visible(lookup_widget("frame24"), FALSE);
@@ -358,7 +468,7 @@ static void siril_app_open(GApplication *application, GFile **files, gint n_file
 					set_seq(path);
 					if (!com.script) {
 						populate_seqcombo(path);
-						set_GUI_CWD();
+						gui_function(set_GUI_CWD, NULL);
 					}
 				}
 				g_free(sequence_dir);
@@ -372,7 +482,7 @@ static void siril_app_open(GApplication *application, GFile **files, gint n_file
 				g_free(image_dir);
 			}
 			if (!com.script)
-				set_GUI_CWD();
+				gui_function(set_GUI_CWD, NULL);
 			open_single_image(path);
 		}
 		g_free(path);
@@ -388,60 +498,88 @@ static void siril_macos_setenv(const char *progname) {
 	gchar resolved_path[PATH_MAX];
 
 	if (realpath(progname, resolved_path)) {
-		gchar *path;
 		gchar tmp[PATH_MAX];
-		gchar *app_dir;
-		gchar lib_dir[PATH_MAX];
-		size_t path_len;
-		struct stat sb;
-		app_dir = g_path_get_dirname(resolved_path);
+		gchar *exe_dir;           /* executable directory */
+		gchar res_dir[PATH_MAX];  /* resources directory  */
+		gchar fra_dir[PATH_MAX];  /* frameworks directory */
 
-		g_snprintf(tmp, sizeof(tmp), "%s/../Resources", app_dir);
-		if (realpath(tmp, lib_dir) && !stat(lib_dir, &sb) && S_ISDIR(sb.st_mode))
-			g_print("Siril is started as MacOS application\n");
-		else
+		exe_dir = g_path_get_dirname(resolved_path);
+
+		/* check if running inside an application bundle and
+		 * set set res_dir and fra_dir accordingly
+		 */
+		g_snprintf(tmp, sizeof(tmp), "%s/../Resources", exe_dir);
+ 	  struct stat sb;
+		if (realpath(tmp, res_dir) && !stat(res_dir, &sb) && S_ISDIR(sb.st_mode)) {
+			g_print("Siril is started as macOS application\n");
+			g_snprintf(tmp, sizeof(tmp), "%s/../Frameworks", exe_dir);
+			realpath(tmp, fra_dir);
+		}
+		else {
+			g_free(exe_dir);
 			return;
+		}
 
-		/* we define the relocated resources path */
-		g_setenv("SIRIL_RELOCATED_RES_DIR", tmp, TRUE);
+		g_mutex_lock(&com.env_mutex);
+		/* store canonical path to resources directory in environment variable. */
+		g_setenv("SIRIL_RELOCATED_RES_DIR", res_dir, TRUE);
 
-		path_len = strlen(g_getenv("PATH") ? g_getenv("PATH") : "")
-			+ strlen(app_dir) + 2;
-		path = g_try_malloc(path_len);
+		/* prepend PATH with our exe_dir (Foo.app/Contents/MacOS) */
+		gchar *path = g_try_malloc(PATH_MAX);
 		if (path == NULL) {
 			g_warning("Failed to allocate memory");
 			exit(EXIT_FAILURE);
 		}
 		if (g_getenv("PATH"))
-			g_snprintf(path, path_len, "%s:%s", app_dir, g_getenv("PATH"));
+			g_snprintf(path, PATH_MAX, "%s:%s", exe_dir, g_getenv("PATH"));
 		else
-			g_snprintf(path, path_len, "%s", app_dir);
-		/* the relocated path is storred in this env. variable in order to be reused if needed */
-		g_free(app_dir);
+			g_snprintf(path, PATH_MAX, "%s", exe_dir);
 		g_setenv("PATH", path, TRUE);
 		g_free(path);
-		g_snprintf(tmp, sizeof(tmp), "%s/share", lib_dir);
+		g_free(exe_dir);
+
+		/* set XDG base directory specification variables */
+		g_snprintf(tmp, sizeof(tmp), "%s/share", res_dir);
 		g_setenv("XDG_DATA_DIRS", tmp, TRUE);
-		g_snprintf(tmp, sizeof(tmp), "%s/share/schemas", lib_dir);
-		g_setenv("GSETTINGS_SCHEMA_DIR", tmp, TRUE);
-		g_snprintf(tmp, sizeof(tmp), "%s/lib/gtk-3.0/3.0.0", lib_dir);
-		g_setenv("GTK_PATH", tmp, TRUE);
-		g_snprintf(tmp, sizeof(tmp), "%s/lib/gdk-pixbuf-2.0/2.10.0/loaders.cache", lib_dir);
-		g_setenv("GDK_PIXBUF_MODULE_FILE", tmp, TRUE);
-		g_snprintf(tmp, sizeof(tmp), "%s/lib/gdk-pixbuf-2.0/2.10.0/loaders", lib_dir);
-		g_setenv("GDK_PIXBUF_MODULE_DIR", tmp, TRUE);
-		g_snprintf(tmp, sizeof(tmp), "%s/etc/fonts", lib_dir);
-		g_setenv("FONTCONFIG_PATH", tmp, TRUE);
-		g_snprintf(tmp, sizeof(tmp), "%s/etc/ca-certificates/cacert.pem", lib_dir);
-		g_setenv("CURL_CA_BUNDLE", tmp, TRUE);
 		if (g_getenv("HOME") != NULL) {
+			g_snprintf (tmp, sizeof(tmp), "%s/Library/Caches/org.siril.Siril", g_getenv("HOME"));
+			g_setenv ("XDG_CACHE_HOME", tmp, TRUE);
 			g_snprintf(tmp, sizeof(tmp), "%s/Library/Application Support/org.siril.Siril", g_getenv("HOME"));
 			g_setenv("XDG_CONFIG_HOME", tmp, TRUE);
-			g_snprintf (tmp, sizeof(tmp), "%s/Library/Caches/org.siril.Siril",
-					g_getenv("HOME"));
-			g_setenv ("XDG_CACHE_HOME", tmp, TRUE);
-
+			g_setenv("XDG_DATA_HOME", tmp, TRUE);
 		}
+
+		/* set GTK related environment variables */
+		g_snprintf(tmp, sizeof(tmp), "%s", fra_dir);
+		g_setenv("GTK_EXE_PREFIX", tmp, TRUE);
+		g_snprintf(tmp, sizeof(tmp), "%s", res_dir);
+		g_setenv("GTK_DATA_PREFIX", tmp, TRUE);
+		g_snprintf(tmp, sizeof(tmp), "%s/etc/loaders.cache", res_dir);
+		g_setenv("GDK_PIXBUF_MODULE_FILE", tmp, TRUE);
+		g_snprintf(tmp, sizeof(tmp), "%s/etc/immodules.cache", res_dir);
+		g_setenv("GTK_IM_MODULE_FILE", tmp, TRUE);
+
+		/* GObject introspection */
+		g_snprintf(tmp, sizeof(tmp), "%s/lib/girepository-1.0", res_dir);
+		g_setenv("GI_TYPELIB_PATH", tmp, TRUE);
+
+		/* set fontconfig related variables */
+		g_snprintf(tmp, sizeof(tmp), "%s/etc/fonts", res_dir);
+		g_setenv("FONTCONFIG_PATH", tmp, TRUE);
+
+		/* set SSL related variables */
+		g_snprintf(tmp, sizeof(tmp), "%s/lib/python3.12/site-packages/certifi/cacert.pem", res_dir);
+		g_setenv("CURL_CA_BUNDLE", tmp, TRUE);
+		g_setenv("SSL_CERT_FILE", tmp, TRUE);
+
+		/* set PYTHONPAH to our bundled packages */
+		g_snprintf(tmp, sizeof(tmp), "%s/lib/python3.12/site-packages", res_dir);
+		g_setenv("PYTHONPATH", tmp, TRUE);
+		g_mutex_unlock(&com.env_mutex);
+
+		/* astropy does not create its director itself */
+		g_snprintf(tmp, sizeof(tmp), "%s/astropy", g_getenv("XDG_CONFIG_HOME"));
+		siril_mkdir_with_parents(tmp, S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH); // perm 755
 	}
 }
 #endif
@@ -473,6 +611,9 @@ int main(int argc, char *argv[]) {
 #elif _WIN32
 	// suppression of annoying error boxes, hack from RawTherapee
 	SetErrorMode(SEM_FAILCRITICALERRORS | SEM_NOGPFAULTERRORBOX | SEM_NOOPENFILEERRORBOX);
+	// avoid Python interfering with Siril's embedded Python
+	g_unsetenv("PYTHONPATH");
+	g_unsetenv("PYTHONHOME");
 #endif
 
 #ifdef _WIN32
@@ -523,7 +664,9 @@ int main(int argc, char *argv[]) {
 		g_printerr("%s\n", help_msg);
 		g_free(help_msg);
 	}
-
+#ifdef HAVE_LIBGIT2
+	git_libgit2_shutdown();
+#endif
 	pipe_stop();		// close the pipes and their threads
 	g_object_unref(app);
 	return status;
