@@ -27,22 +27,28 @@
 #include <iostream>
 #include <iomanip>
 #include <opencv2/core/core.hpp>
-#include <opencv2/imgproc/imgproc.hpp>
-#include "opencv2/core/version.hpp"
+#include <opencv2/core/matx.hpp>
 #define CV_RANSAC FM_RANSAC
 #include <opencv2/calib3d.hpp>
 
+// for mosaics
+#include <opencv2/stitching/warpers.hpp>
+
 #include "core/siril.h"
+#include "core/proto.h"
 #include "core/siril_log.h"
+#include "core/settings.h"
 #include "registration/registration.h"
-#include "registration/matching/misc.h"
 #include "registration/matching/atpmatch.h"
 #include "opencv.h"
+#include "guidedfilter.h"
 
 #ifdef __cplusplus
 extern "C" {
 #endif
+#include "io/image_format_fits.h"
 #include "algos/statistics.h"
+#include "registration/distorsion.h"
 #ifdef __cplusplus
 }
 #endif
@@ -52,7 +58,7 @@ extern "C" {
 
 using namespace cv;
 
-static void convert_H_to_MatH(Homography *from, Mat &to);
+static void convert_H_to_MatH(const Homography *from, Mat &to);
 static void convert_MatH_to_H(Mat from, Homography *to);
 
 /* TODO:
@@ -156,7 +162,9 @@ static int Mat_to_image(fits *image, Mat *in, Mat *out, void *bgr, int target_rx
 	if (bgr) free(bgr);
 	if (image->naxes[2] == 1) {
 		free(image->data);
+		image->data = NULL; // This must be set to NULL as the function can return on error before it is reassigned to point at the modified data
 		free(image->fdata);
+		image->fdata = NULL; // This must be set to NULL as the function can return on error before it is reassigned to point at the modified data
 	}
 
 	size_t ndata = target_rx * target_ry;
@@ -217,7 +225,7 @@ static int Mat_to_image(fits *image, Mat *in, Mat *out, void *bgr, int target_rx
 			channel[1].release();
 			channel[2].release();
 		} else {
-			image->fdata = (float *)out->data;
+			image->fdata = (float *) out->data;
 			image->fpdata[RLAYER] = image->fdata;
 			image->fpdata[GLAYER] = image->fdata;
 			image->fpdata[BLAYER] = image->fdata;
@@ -314,11 +322,11 @@ void cvTransformImageRefPoint(Homography Hom, point refpointin, point *refpointo
 	Mat H = Mat(3, 3, CV_64FC1);
 	convert_H_to_MatH(&Hom, H);
 	refptout = H * Mat(refptin);
-	refpointout->x = refptout.at<double>(0, 0);
-	refpointout->y = refptout.at<double>(0, 1);
+	refpointout->x = refptout.at<double>(0) / refptout.at<double>(2);
+	refpointout->y = refptout.at<double>(1) / refptout.at<double>(2);
 }
 
-static void convert_H_to_MatH(Homography *from, Mat &to) {
+static void convert_H_to_MatH(const Homography *from, Mat &to) {
 	to.at<double>(0, 0) = from->h00;
 	to.at<double>(0, 1) = from->h01;
 	to.at<double>(0, 2) = from->h02;
@@ -344,10 +352,10 @@ static void convert_MatH_to_H(Mat from, Homography *to) {
 
 void cvGetEye(Homography *Hom) {
 	Mat M = Mat::eye(3, 3, CV_64FC1);
-	convert_MatH_to_H(M, Hom);
+	convert_MatH_to_H(std::move(M), Hom);
 }
 
-void cvTransfPoint(double *x, double *y, Homography Href, Homography Himg) {
+void cvTransfPoint(double *x, double *y, Homography Href, Homography Himg, double scale) {
 	Mat_<double> ref(3,1);
 	Mat_<double> dst;
 	Mat H0 = Mat(3, 3, CV_64FC1);
@@ -360,25 +368,29 @@ void cvTransfPoint(double *x, double *y, Homography Href, Homography Himg) {
 	convert_H_to_MatH(&Himg, H1);
 	if (cv::determinant(H1) == 0) return;
 	dst = H1.inv() * H0 * ref;
-	*x = dst(0,0);
-	*y = dst(1,0);
+	if (scale != 1.) {
+		Mat S = Mat::eye(3, 3, CV_64FC1);
+		S.at<double>(0,0) = scale;
+		S.at<double>(1,1) = scale;
+		dst = S * dst;
+	}
+	*x = dst(0,0) / dst(2,0);
+	*y = dst(1,0) / dst(2,0);
 }
 
-void cvTransfH(Homography Href, Homography Himg, Homography *Hres) {
+void cvTransfH(Homography *Href, Homography *Himg, Homography *Hres) {
 	Mat_<double> ref(3,1);
 	Mat_<double> dst;
 	Mat H0 = Mat(3, 3, CV_64FC1);
 	Mat H1 = Mat(3, 3, CV_64FC1);
 	Mat H2 = Mat(3, 3, CV_64FC1);
 
-	convert_H_to_MatH(&Href, H0);
-	convert_H_to_MatH(&Himg, H1);
+	convert_H_to_MatH(Href, H0);
+	convert_H_to_MatH(Himg, H1);
 	H2 = H1.inv() * H0;
-	convert_MatH_to_H(H2, Hres);
+	convert_MatH_to_H(std::move(H2), Hres);
 }
 
-// TODO: to be looked into when we sort out the conventions for astrometry (see #1103)
-
 unsigned char *cvCalculH(s_star *star_array_img,
 		struct s_star *star_array_ref, int n, Homography *Hom, transformation_type type, float offset) {
 
@@ -393,7 +405,7 @@ unsigned char *cvCalculH(s_star *star_array_img,
 	unsigned char *ret = NULL;
 
 	/* 	build vectors with lists of stars.
-		When defined with offset, the -0.5 term comes from the difference 
+		When defined with offset, the -0.5 term comes from the difference
 		in convention between how we compute PSF
 		and opencv (zero coordinate at edge of pixel vs at center) */
 	switch (type) {
@@ -458,29 +470,26 @@ unsigned char *cvCalculH(s_star *star_array_img,
 		return NULL;
 	}
 
-	convert_MatH_to_H(H, Hom);
+	convert_MatH_to_H(std::move(H), Hom);
 
 	mask.release();
 	return ret;
 }
 
-// transform an image using the homography.
-int cvTransformImage(fits *image, unsigned int width, unsigned int height, Homography Hom, gboolean upscale2x, int interpolation, gboolean clamp) {
-	Mat in, out;
-	void *bgr = NULL;
-	int target_rx = width, target_ry = height;
-	int source_ry = image->ry;
-
-	if (image_to_Mat(image, &in, &out, &bgr, target_rx, target_ry))
-		return 1;
-
-	Mat H = Mat(3, 3, CV_64FC1);
-	convert_H_to_MatH(&Hom, H);
-
-	if (upscale2x) {
+static void cvPrepareH(Mat H, double scale, int source_rx, int source_ry, int target_rx, int target_ry) {
+	if (scale != 1.) {
 		Mat S = Mat::eye(3, 3, CV_64FC1);
-		S.at<double>(0,0) = 2.0;
-		S.at<double>(1,1) = 2.0;
+		S.at<double>(0,0) = scale;
+		S.at<double>(1,1) = scale;
+		// the terms below are the corrections to make the scaling about the image center (and not top left)
+		// it's the result of -T2^-1*So*T1
+		// where T1 and T2 are the shifts to the center in original and scaled images 
+		// using opencv convention, so (rx/2-0.5;ry/2-0.5) and (target_rx/2-0.5;target_ry/2-0.5)
+		// and So the scale about top left origin which is simply [s 0 0][0 s 0][0 0 1]
+		// rx/ry are the sizes of the area to be projected, that is dst_rx/scale and dst_ry/scale
+		// We then simplify as 0.5 * (1 - scale)
+		S.at<double>(0,2) -= 0.5 * (1 - scale);
+		S.at<double>(1,2) -= 0.5 * (1 - scale);
 		H = S * H;
 	}
 
@@ -494,26 +503,116 @@ int cvTransformImage(fits *image, unsigned int width, unsigned int height, Homog
 	F2.at<double>(1,2) = target_ry - 1.0;
 
 	H = F2.inv() * H * F1;
+}
 
-	// OpenCV function
-	warpPerspective(in, out, H, Size(target_rx, target_ry), interpolation, BORDER_TRANSPARENT);
+void cvPrepareDrizzleH(Homography *Hom, double scale, int source_rx, int source_ry, int target_rx, int target_ry) {
+	Mat H = Mat(3, 3, CV_64FC1);
+	convert_H_to_MatH(Hom, H);
+	cvPrepareH(H, scale, source_rx, source_ry, target_rx, target_ry);
+	convert_MatH_to_H(std::move(H), Hom);
+}
+
+// transform an image using the homography.
+int cvTransformImage(fits *image, unsigned int width, unsigned int height, Homography Hom, float scale, int interpolation, gboolean clamp, disto_data *disto) {
+	Mat in, out;
+	void *bgr = NULL;
+	int target_rx = width, target_ry = height;
+
+	if (image_to_Mat(image, &in, &out, &bgr, target_rx, target_ry))
+		return 1;
+
+	Mat H = Mat(3, 3, CV_64FC1);
+	convert_H_to_MatH(&Hom, H);
+	cvPrepareH(H, scale, image->rx, image->ry, target_rx, target_ry);
+
+	// no distortion case
+	if (!disto || (disto->dtype != DISTO_MAP_D2S && disto->dtype != DISTO_D2S)) {
+		// OpenCV function
+		warpPerspective(in, out, H, Size(target_rx, target_ry), interpolation, BORDER_TRANSPARENT);
+		if ((interpolation == OPENCV_LANCZOS4 || interpolation == OPENCV_CUBIC) && clamp) {
+			Mat guide, tmp1;
+			init_guide(image, target_rx, target_ry, &guide);
+			// Create guide image
+			warpPerspective(in, guide, H, Size(target_rx, target_ry), OPENCV_AREA, BORDER_TRANSPARENT);
+			tmp1 = (out < guide * CLAMPING_FACTOR);
+			Mat element = getStructuringElement( MORPH_ELLIPSE,
+						Size(3, 3), Point(-1,-1));
+			dilate(tmp1, tmp1, element);
+
+			copyTo(guide, out, tmp1); // Guide copied to the clamped pixels
+			guide.release();
+			tmp1.release();
+		}
+		return Mat_to_image(image, &in, &out, bgr, target_rx, target_ry);
+	}
+
+	// distortion case - we need to compute maps to pass to remapping to avoid double interpolation
+	float *xmap, *ymap;
+	Mat Hinv = H.inv();  // dst->src
+	xmap = (float *)malloc(target_rx * target_ry *sizeof(float));
+	ymap = (float *)malloc(target_rx * target_ry *sizeof(float));
+	if (!xmap || !ymap) {
+		free(xmap);
+		free(ymap);
+		PRINT_ALLOC_ERR;
+		return 2;
+	}
+	prepare_H_with_disto_4remap(Hinv.ptr<double>(0), image->rx, image->ry, target_rx, target_ry, disto, xmap, ymap);
+	Mat uxmap = Mat(target_ry, target_rx, CV_32FC1, xmap);
+	Mat uymap = Mat(target_ry, target_rx, CV_32FC1, ymap);
+	remap(in, out, uxmap, uymap, interpolation, BORDER_TRANSPARENT);
 	if ((interpolation == OPENCV_LANCZOS4 || interpolation == OPENCV_CUBIC) && clamp) {
 		Mat guide, tmp1;
 		init_guide(image, target_rx, target_ry, &guide);
 		// Create guide image
-		warpPerspective(in, guide, H, Size(target_rx, target_ry), OPENCV_AREA, BORDER_TRANSPARENT);
+		remap(in, guide, uxmap, uymap, OPENCV_AREA, BORDER_TRANSPARENT);
 		tmp1 = (out < guide * CLAMPING_FACTOR);
 		Mat element = getStructuringElement( MORPH_ELLIPSE,
-                       Size(3, 3), Point(-1,-1));
+					Size(3, 3), Point(-1,-1));
 		dilate(tmp1, tmp1, element);
 
 		copyTo(guide, out, tmp1); // Guide copied to the clamped pixels
 		guide.release();
 		tmp1.release();
 	}
-	return Mat_to_image(image, &in, &out, bgr, target_rx, target_ry);
+	free(xmap);
+	free(ymap);
+	return Mat_to_image(image, &in, &out, bgr, width, height);
 }
 
+void cvDownscaleBlendMask(int rx, int ry, int out_rx, int out_ry, uint8_t *maskin, float *maskout) {
+	Mat _maskin = Mat(ry, rx, CV_8U, maskin);
+	Mat _maskindown = Mat(out_ry + 2, out_rx + 2, CV_8U, Scalar(0));
+	Mat _maskoutdown32 = Mat(out_ry + 2, out_rx + 2, CV_32F, Scalar(0.));
+	Mat _maskout = Mat(out_ry, out_rx, CV_32F, maskout);
+	int morph_size = 3;
+	// first we dilate and erode to make sure we fill holes left by drizzling if any
+	Mat element = getStructuringElement(MORPH_RECT,
+			Size(2 * morph_size + 1, 2 * morph_size + 1),
+			Point(morph_size, morph_size));
+  	dilate(_maskin, _maskin, element);
+	erode(_maskin, _maskin, element);
+	// we leave a border of one pixel black to make sure the distance transform
+	// will later count distance from this border (if the initial mask was entirely white for instance)
+	// we get a handle to the inner part of the maskout (without the border)
+	Mat _maskindownroi = _maskindown(Rect(1, 1, out_rx, out_ry));
+	// we resize
+	resize(_maskin, _maskindownroi, _maskindownroi.size(), 0, 0, INTER_LINEAR);
+	// we compute the distances, it returns a 32b array
+	distanceTransform(_maskindown, _maskoutdown32, DIST_L2, 3, CV_32F);
+	Mat _maskoutdownroi = _maskoutdown32(Rect(1, 1, out_rx, out_ry));
+	_maskoutdownroi.copyTo(_maskout);
+}
+
+void cvUpscaleBlendMask(int rx, int ry, int out_rx, int out_ry, float *maskin, float *maskout) {
+	Mat _maskin = Mat(ry, rx, CV_32F, maskin);
+	Mat _maskout = Mat(out_ry, out_rx, CV_32F, maskout);
+	resize(_maskin, _maskout, _maskout.size(), 0, 0, INTER_LINEAR);
+	flip(_maskout, _maskout, 0);  // don't know why we need to flip but does not work without
+}
+
+
+
 int cvUnsharpFilter(fits* image, double sigma, double amount) {
 	Mat in, out;
 	void *bgr = NULL;
@@ -538,6 +637,55 @@ int cvUnsharpFilter(fits* image, double sigma, double amount) {
 	return Mat_to_image(image, &in, &out, bgr, target_rx, target_ry);
 }
 
+int cvBilateralFilter(fits* image, double d, double sigma_col, double sigma_space) {
+	Mat in, out;
+	void *bgr = NULL;
+	if (image_to_Mat(image, &in, &out, &bgr, image->rx, image->ry))
+		return 1;
+	siril_debug_print("using opencv Bilateral Filter (CPU)\n");
+	bilateralFilter(in, out, d, sigma_col, sigma_space, BORDER_DEFAULT);
+	return Mat_to_image(image, &in, &out, bgr, image->rx, image->ry);
+}
+
+int cvGuidedFilter(fits* image, fits *guide, double r, double eps) {
+	Mat in, out, guide_mat;
+	void *bgr = NULL;
+	int rx = guide->rx, ry = guide->ry;
+
+	if (image_to_Mat(image, &in, &out, &bgr, image->rx, image->ry))
+		return 1;
+	if (image == guide) {
+		guide_mat = in.clone();
+	} else {
+		if (guide->type == DATA_USHORT) {
+			if (guide->naxes[2] == 1) {
+				guide_mat = Mat(ry, rx, CV_16UC1, guide->data);
+			} else if (guide->naxes[2] == 3) {
+				WORD *bgr_u = fits_to_bgrbgr_ushort(guide);
+				if (!bgr_u) return -1;
+				guide_mat = Mat(ry, rx, CV_16UC3, bgr_u);
+			}
+		}
+		else if (guide->type == DATA_FLOAT) {
+			if (guide->naxes[2] == 1) {
+				guide_mat = Mat(ry, rx, CV_32FC1, guide->fdata);
+			}
+			else if (guide->naxes[2] == 3) {
+				float *bgr_f = fits_to_bgrbgr_float(guide);
+				if (!bgr_f) return -1;
+				guide_mat = Mat(ry, rx, CV_32FC3, bgr_f);
+			}
+		}
+	}
+	if (guide_mat.channels() != in.channels())
+		cvtColor(guide_mat, guide_mat, COLOR_GRAY2BGR);
+	siril_debug_print("using Guided Filter (CPU)\n");
+	Mat result = guidedFilter(guide_mat, in, r, eps, -1);
+	result.copyTo(out);
+	guide_mat.release();
+	return Mat_to_image(image, &in, &out, bgr, image->rx, image->ry);
+}
+
 /* Work on grey images. If image is in RGB it must be first converted
  * in CieLAB. Then, only the first channel is applied
  */
@@ -856,7 +1004,7 @@ double cvCalculRigidTransform(s_star *star_array_in,
 	}
 
 	Hom->Inliers = n;
-	convert_MatH_to_H(H, Hom);
+	convert_MatH_to_H(std::move(H), Hom);
 
 	return err;
 
@@ -869,7 +1017,7 @@ void cvMultH(Homography H1, Homography H2, Homography *Hout) {
 	convert_H_to_MatH(&H1, _H1);
 	convert_H_to_MatH(&H2, _H2);
 	_H = _H1 * _H2;
-	convert_MatH_to_H(_H, Hout);
+	convert_MatH_to_H(std::move(_H), Hout);
 }
 
 void cvGetMatrixReframe(double x, double y, int w, int h, double angle, Homography *Hom) {
@@ -901,52 +1049,23 @@ void cvGetMatrixReframe(double x, double y, int w, int h, double angle, Homograp
 	H = H.inv();
 	std::cout << H << std::endl;
 
-	convert_MatH_to_H(H, Hom);
-}
-
-void cvGetMatrixResize(double cxin, double cyin, double cxout, double cyout, double scale, Homography *Hom) {
-
-	// shift to get to the center of the initial image
-	Mat S =  Mat::eye(3, 3, CV_64FC1);
-	S.at<double>(0, 2) = cxin;
-	S.at<double>(1, 2) = cyin;
-
-	// shift backwards to set the top-left point of the final image
-	Mat S2 =  Mat::eye(3, 3, CV_64FC1);
-	S2.at<double>(0, 2) = -cxout;
-	S2.at<double>(1, 2) = -cyout;
-
-	// get scaling matrix about origin {0, 0}
-	Mat H = Mat::eye(3, 3, CV_64FC1);
-	H.at<double>(0, 0) = scale;
-	H.at<double>(1, 1) = scale;
-	// std::cout << H << std::endl;
-
-	H = S * H * S2;
-	std::cout << H << std::endl;
-
-	// transform is final to orginal, we need to inverse
-	// to have H from original to final
-	H = H.inv();
-	std::cout << H << std::endl;
-
-	convert_MatH_to_H(H, Hom);
+	convert_MatH_to_H(std::move(H), Hom);
 }
 
 void cvGetBoundingRectSize(fits *image, point center, double angle, int *w, int *h) {
 	Rect frame;
 	Point2f pt(center.x, center.y);
-	frame = RotatedRect(pt, Size(image->rx, image->ry), angle).boundingRect();
+	frame = RotatedRect(pt, Size(image->rx, image->ry), angle).boundingRect2f();
 	siril_debug_print("after rotation, new image size will be %d x %d\n", frame.width, frame.height);
-	*w = frame.width;
-	*h = frame.height;
+	*w = (int)frame.width;
+	*h = (int)frame.height;
 }
 
 void cvInvertH(Homography *Hom) {
 	Mat H = Mat(3, 3, CV_64FC1);
 	convert_H_to_MatH(Hom, H);
 	H = H.inv();
-	convert_MatH_to_H(H, Hom);
+	convert_MatH_to_H(std::move(H), Hom);
 	// std::cout << "H" << std::endl;
 	// std::cout << H << std::endl;
 }
@@ -957,12 +1076,133 @@ void cvApplyFlips(Homography *Hom, int source_ry, int target_ry) {
 	/* modify matrix for reverse Y axis */
 	Mat F1 = Mat::eye(3, 3, CV_64FC1);
 	F1.at<double>(1,1) = -1.0;
-	F1.at<double>(1,2) = source_ry - 1.0;
+	F1.at<double>(1,2) = source_ry;
 
 	Mat F2 = Mat::eye(3, 3, CV_64FC1);
 	F2.at<double>(1,1) = -1.0;
-	F2.at<double>(1,2) = target_ry - 1.0;
+	F2.at<double>(1,2) = target_ry;
 
 	H = F2.inv() * H * F1;
-	convert_MatH_to_H(H, Hom);
+	convert_MatH_to_H(std::move(H), Hom);
 }
+
+
+// Used to convert a H matrix written in display convention to opencv convention
+void cvdisplay2ocv(Homography *Hom) {
+	Mat H = Mat(3, 3, CV_64FC1);
+	convert_H_to_MatH(Hom, H);
+
+	/* modify matrix to go to opencv convention */
+	Mat S1 = Mat::eye(3, 3, CV_64FC1);
+	S1.at<double>(0,2) = 0.5;
+	S1.at<double>(1,2) = 0.5;
+
+	H = S1.inv() * H * S1;
+	convert_MatH_to_H(std::move(H), Hom);
+}
+
+/*
+Takes an angle in degrees and a rotationtype (X, Y or Z)
+Fills the 3D rotation matrix
+Returns FALSE on error
+*/
+static gboolean cvRotMat1(double angle, rotation_type rottype, Mat &R) {
+	angle *= G_PI / 180.;
+	double ca = cos(angle);
+	double sa = sin(angle);
+	gboolean retval = TRUE;
+
+	switch (rottype) {
+		case ROTX:
+			R.at<double>(1,1) = ca;
+			R.at<double>(2,2) = ca;
+			R.at<double>(1,2) = -sa;
+			R.at<double>(2,1) = sa;
+			break;
+		case ROTY:
+			R.at<double>(0,0) = ca;
+			R.at<double>(2,2) = ca;
+			R.at<double>(2,0) = -sa;
+			R.at<double>(0,2) = sa;
+			break;
+		case ROTZ:
+			R.at<double>(0,0) = ca;
+			R.at<double>(1,1) = ca;
+			R.at<double>(0,1) = -sa;
+			R.at<double>(1,0) = sa;
+			break;
+		default:
+			retval = FALSE;
+			break;
+	}
+	return retval;
+}
+
+
+/*
+Takes an angle array[3] in degrees and a rotationtype vector[3] (X, Y or Z)
+Fills the 3D rotation matrix
+The angles are applied in the input order
+The flag W2C specifies if the matrix out is World-to-Camera or Camera-to-World
+Returns FALSE on error
+*/
+gboolean cvRotMat3(double angles[3], rotation_type rottype[3], gboolean W2C, Homography *R) {
+	Mat _R = Mat::eye(3, 3, CV_64FC1);
+	gboolean retval = TRUE;
+	for (int i = 0; i < 3; i++) {
+		Mat M = Mat::eye(3, 3, CV_64FC1);
+		if (!cvRotMat1(angles[i], rottype[i], M)) {
+			retval = FALSE;
+			break;
+		}
+		_R = M * _R; // left-multiply as the full matrix is R3*R2*R1
+	}
+	if (W2C)
+		_R = _R.t(); // if we need W2C, we need to invert (i.e. transpose for R mats)
+	convert_MatH_to_H(std::move(_R), R);
+	// std::cout << R << std::endl;
+	return retval;
+}
+
+// Computes the relative rotation matrix between Rref and R
+void cvRelRot(Homography *Ref, Homography *R, Homography *Rout) {
+	Mat _Ref = Mat(3, 3, CV_64FC1);
+	Mat _R = Mat(3, 3, CV_64FC1);
+	Mat _H = Mat(3, 3, CV_64FC1);
+	Mat _Rout = Mat(3, 3, CV_64FC1);
+	convert_H_to_MatH(Ref, _Ref);
+	convert_H_to_MatH(R, _R);
+	_Rout = _Ref.t() * _R;
+	convert_MatH_to_H(std::move(_Rout), Rout);
+}
+
+// Computes Homography from cameras R and K
+void cvcalcH_fromKKR(Homography *Kref, Homography *K, Homography *R, Homography *H) {
+	Mat _Kref = Mat(3, 3, CV_64FC1);
+	Mat _K = Mat(3, 3, CV_64FC1);
+	Mat _R = Mat(3, 3, CV_64FC1);
+	Mat _H = Mat(3, 3, CV_64FC1);
+	convert_H_to_MatH(Kref, _Kref);
+	convert_H_to_MatH(K, _K);
+	convert_H_to_MatH(R, _R);
+
+	//Compute H and returning
+	_H = _Kref * _R * _K.inv();
+	convert_MatH_to_H(std::move(_H), H);
+}
+
+int cvCalcH_from_corners(double *x_img, double *y_img, double *x_ref, double *y_ref, Homography *Hom) {
+	std::vector<Point2f> ref;
+	std::vector<Point2f> img;
+
+	for (int i = 0; i < 4; i++) {
+		ref.push_back(Point2f(x_ref[i], y_ref[i]));
+		img.push_back(Point2f(x_img[i], y_img[i]));
+	}
+
+	Mat H = findHomography(img, ref, 0);
+	if (H.empty())
+		return -1;
+	convert_MatH_to_H(std::move(H), Hom);
+	return 0;
+}
\ No newline at end of file
